// Ionic Starter App

// angular.module is a global place for creating, registering and retrieving Angular modules
// 'starter' is the name of this angular module example (also set in a <body> attribute in index.html)
// the 2nd parameter is an array of 'requires'
// 'starter.controllers' is found in controllers.js

window.appRootDirName = "download_test";
// document.addEventListener("deviceready", function() {
//     console.log("device is ready");

// }, false);

angular.module('vmb', ['ionic', 'vmb.controllers'])
    .factory('$auth', ['$rootScope', '$http',
        function($rootScope, $http) {
            var user;
            if (window.sessionStorage.user) {
                try {
                    user = JSON.parse(window.sessionStorage.user);
                } catch (e) {}
            } else {
                user = {
                    id: 0,
                    firstname: 'Guest',
                    role: 'guest'
                };
            }
            $rootScope.user = user;

            function fetchUser() {
                return $http.get('/api/auth/user').then(function(res) {
                    $rootScope.user = res.data;
                    window.sessionStorage.user = JSON.stringify(res.data);
                });
            }

            var initialUserFetch = fetchUser();
            return {
                ready: function() {
                    return initialUserFetch;
                },
                login: function(email, password) {
                    return $http.post('/api/auth/login', {
                        email: email,
                        password: password
                    }).then(function(res) {
                        $rootScope.API_TOKEN = window.sessionStorage.API_TOKEN = res.data.token;
                        console.log('registering device ' + window.deviceToken);
                        $http.post('/api/device/register', {
                            token: window.deviceToken,
                            device_type: window.deviceType
                        }).success(function(res){
                            console.log('registered ', JSON.stringify(res));
                        }).error(function(err){
                            console.log('register error ', JSON.stringify(err));
                        });
                        return fetchUser();
                    });
                },
                logout: function() {
                    return $http.post('/api/auth/logout').then(function() {
                        return fetchUser();
                    });
                },
                isAuthenticated: function() {
                    return !!this.getSession().id;
                },
                isAuthorized: function(authorizedRoles) {
                    if (!angular.isArray(authorizedRoles)) {
                        authorizedRoles = [authorizedRoles];
                    }
                    return (this.isAuthenticated() &&
                        authorizedRoles.indexOf(this.getSession().role) !== -1);
                },
                getSession: function() {
                    return $rootScope.user;
                },
                getDefaultHome: function() {
                    var defHome = {
                        'user': 'vmailbox',
                        'agent': 'agent',
                        'staff': 'staff',
                        'admin': 'Admin'
                    };
                    return defHome[this.getSession().role];
                }
            };
        }
    ]).factory('$authInterceptor', ['$rootScope', '$window',
        function($rootScope, $window) {
            return {
                request: function(config) {
                    if (config.url.indexOf('/app') === 0) {
                        config.url.splice(0, 4);
                        return config;
                    }
                    if (config.url[0] === '/' && config.url.slice(-5) !== '.html') {
                        config.url = window.__config.API_BASE_URL + config.url;
                        if ($rootScope.API_TOKEN || $window.sessionStorage.API_TOKEN) {
                            config.headers['AUTH-TOKEN'] = $rootScope.API_TOKEN || $window.sessionStorage.API_TOKEN;
                        }
                    }

                    return config;
                },
                response: function(response) {
                    return response;
                }
            };
        }
    ])
    .run(function($ionicPlatform, $rootScope, $ionicPopup, $http) {
        $ionicPlatform.ready(function() {
            // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
            // for form inputs)
            if (window.cordova && window.cordova.plugins.Keyboard) {
                cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
            }
            if (window.StatusBar) {
                // org.apache.cordova.statusbar required
                StatusBar.styleDefault();
            }

            window.onNotificationAPN = function(event) {
                $ionicPopup.alert({
                    title: 'New Mail',
                    template: 'You\'ve got a new Mail.'
                });
                if (event.alert) {
                    navigator.notification.alert(event.alert);
                }

                if (event.sound) {
                    var snd = new Media(event.sound);
                    snd.play();
                }

                if (event.badge) {
                    pushNotification.setApplicationIconBadgeNumber(successHandler, errorHandler, event.badge);
                }
            };

            window.onNotification = function(e) {
                console.log('onNotification: ' + JSON.stringify(e));
                switch (e.event) {
                    case 'registered':
                        if (e.regid.length > 0) {
                            console.log('<li>REGISTERED -> REGID:' + e.regid + "</li>");
                            // Your GCM push server needs to know the regID before it can push to this device
                            // here is where you might want to send it the regID for later use.
                            console.log("regID = " + e.regid);
                            window.deviceToken = e.regid;
                            window.deviceType = 'android';
                        }
                        break;

                    case 'message':
                        // if this flag is set, this notification happened while we were in the foreground.
                        // you might want to play a sound to get the user's attention, throw up a dialog, etc.
                        if (e.foreground) {
                            $ionicPopup.alert({
                                title: 'New Mail',
                                template: 'You\'ve got a new Mail.'
                            });
                        } else { // otherwise we were launched because the user touched a notification in the notification tray.
                            if (e.coldstart) {
                                $ionicPopup.alert({
                                    title: 'New Mail',
                                    template: 'You\'ve got a new Mail.'
                                });
                            } else {
                                $ionicPopup.alert({
                                    title: 'New Mail',
                                    template: 'You\'ve got a new Mail.'
                                });
                            }
                        }
                        break;

                    case 'error':
                        console.log('error');
                        break;

                    default:
                        console.log('default');
                        break;
                }
            };

            window.errorHandler = function(e) {
                console.log('error' + JSON.stringify(e));
            };

            if (window.cordova && window.plugins.pushNotification) {
                window.pushNotification = window.plugins.pushNotification;

                var device = ionic.Platform.device();
                if (device.platform) {
                    if (device.platform == 'android' || device.platform == 'Android' || device.platform == "amazon-fireos") {
                        pushNotification.register(
                            function(e) {
                                console.log('got token: ' + JSON.stringify(e));
                            },
                            errorHandler, {
                                "senderID": "273769283379",
                                "ecb": "onNotification"
                            });
                    } else {
                        pushNotification.register(
                            function(e) {
                                console.log('got token: ' + JSON.stringify(e));
                                window.deviceToken = e;
                                window.deviceType = 'ios';
                            },
                            errorHandler, {
                                "badge": "true",
                                "sound": "true",
                                "alert": "true",
                                "ecb": "onNotificationAPN"
                            });
                    }
                }
            }

            if (window.cordova) {
                window.requestFileSystem(LocalFileSystem.PERSISTENT, 0, function(fileSystem) {
                    console.log("filesystem gotter");
                    window.fileSystem = fileSystem;
                    fileSystem.root.getDirectory(window.appRootDirName, {
                        create: true,
                        exclusive: false
                    }, function(entry) {
                        window.appRootDir = entry;
                        console.log("application dir is ready");
                    }, function() {
                        console.log("failed to get filesystem");
                    });
                }, function() {
                    console.log("failed to get filesystem");
                });
            }

            $rootScope.API_BASE_URL = window.__config.API_BASE_URL;
            $rootScope.switchLanguage = function(lang) {
                console.log('Changing to  language ...', lang);
                gettextCatalog.setCurrentLanguage(lang);
                gettextCatalog.loadRemote($rootScope.API_BASE_URL + '/translations/' + lang + '.json');
            };
            $rootScope.$on('$stateChangeStart', function(event, next, nextParams) {
                if (!next.data) {
                    return;
                }
                // authorizedRoles are stored in the state info
                var authorizedRoles = next.data.authorizedRoles;
                if (!$auth.isAuthorized(authorizedRoles)) {
                    event.preventDefault();
                    if ($auth.isAuthenticated()) {
                        // user is not allowed
                        $rootScope.$broadcast('notAuthorized');
                    } else {
                        // user is not logged in
                        var cancel = $rootScope.$on('loginSuccess', function() {
                            cancel();
                            $state.go(next.name, nextParams);
                        });
                        $rootScope.$broadcast('notAuthenticated');
                        $state.go('login');
                    }
                }
            });
        });
    })

.config(function($stateProvider, $urlRouterProvider, $httpProvider) {
    $httpProvider.defaults.withCredentials = false;
    $httpProvider.interceptors.push('$authInterceptor');
    delete $httpProvider.defaults.headers.common['X-Requested-With'];

    $stateProvider
        .state('app', {
            url: "/app",
            abstract: true,
            templateUrl: "templates/menu.html",
            controller: 'AppCtrl'
        })

    .state('app.search', {
            url: "/search",
            views: {
                'menuContent': {
                    templateUrl: "templates/search.html",
                    controller: "SearchCtrl"
                }
            }
        })
        .state('app.mailbox', {
            url: "/mailbox/:mailbox",
            views: {
                'menuContent': {
                    templateUrl: "templates/mailbox.html",
                    controller: 'MailboxCtrl'
                }
            }
        })
        .state('app.mail', {
            url: "/mail/:mailId",
            views: {
                'menuContent': {
                    templateUrl: "templates/mail.html",
                    controller: 'MailCtrl'
                }
            }
        })
        .state('app.forward', {
            url: "/mail/:mailId/forward",
            views: {
                'menuContent': {
                    templateUrl: "templates/ForwardRequest.html",
                    controller: 'MailCtrl'
                }
            }
        });
    // if none of the above states are matched, use this as the fallback
    $urlRouterProvider.otherwise('/app/mailbox/Inbox');
});
