angular.module('vmb.controllers', [])

.controller('AppCtrl', function($scope, $ionicModal, $state, $auth, $vmb, $q) {
    // Form data for the login modal
    $scope.loginData = {};
    var foldersReadyDefer = $q.defer();
    $scope.foldersReady = foldersReadyDefer.promise;
    // Create the login modal that we will use later
    var loginDialogReady = $ionicModal.fromTemplateUrl('templates/login.html', {
        scope: $scope
    }).then(function(modal) {
        $scope.modal = modal;
    });

    // Triggered in the login modal to close it
    $scope.closeLogin = function() {
        if (!$scope.isAuthenticated) {
            $scope.login();
        } else {
            $scope.modal.hide();
        }
    };

    // Open the login modal
    $scope.login = function() {
        $scope.modal.show();
    };

    // Perform the login action when the user submits the login form
    $scope.doLogin = function() {
        console.log('Doing login', $scope.loginData);
        $auth.login($scope.loginData.email, $scope.loginData.password).then(onLogin, function(res) {
            console.log(JSON.stringify(res));
            $scope.loginData.error = 'Invalid Credentials';
        });
    };

    $scope.logout = function() {
        $auth.logout().then(function() {
            $scope.login();
        });
    };

    $scope.$on('modal.hidden', function() {
        if (!$scope.isAuthenticated) {
            setTimeout(function() {
                $scope.login();
                $scope.$apply();
            }, 1000);
        }
    });

    function onLogin() {
        $scope.isAuthenticated = $auth.isAuthenticated();
        $vmb.getFolders().success(function(res) {
            $scope.folders = res;
        }).then(function() {
            var inbox = $scope.folders.filter(function(f) {
                return f.name === 'Inbox';
            })[0];
            $scope.currentFolderId = inbox.id;
            $state.go('app.mailbox', {
                mailbox: $scope.currentFolderId
            });
            $scope.closeLogin();
            foldersReadyDefer.resolve();
        });
    }

    $auth.ready().then(function() {
        $scope.isAuthenticated = $auth.isAuthenticated();
        if (!$scope.isAuthenticated) {
            loginDialogReady.then(function() {
                $scope.login();
            });
        } else {
            onLogin();
        }
    });

})

.controller('MailboxCtrl', function($scope, $state, $vmb, $ionicPopover) {
    $scope.mailboxId = $state.params.mailbox;
    $scope.from = 0;
    $scope.pageSize = 20;
    $scope.mails = [];
    $scope.$parent.foldersReady.then(function() {
        $scope.mailbox = $scope.$parent.folders.filter(function(f) {
            return f.id == $scope.mailboxId || f.name == $scope.mailboxId;
        })[0];
        $vmb.getMails($scope.mailbox.id, $scope.from, $scope.pageSize).then(function(res) {
            $scope.mails = $scope.mails.concat(res.data.mails);
            $scope.totalItems = res.data.paging;
            $scope.$broadcast('scroll.infiniteScrollComplete');
        });
    });

    $scope.openMail = function(m) {
        $scope.$parent.selectedMail = m;
        $state.go("app.mail", {
            mailId: m.id
        });
    };

    $scope.moreDataCanBeLoaded = function() {
        return $scope.from + $scope.pageSize < $scope.totalItems;
    };

    $scope.loadMoreData = function() {
        if (!$scope.moreDataCanBeLoaded()) {
            $scope.$broadcast('scroll.infiniteScrollComplete');
            return;
        }
        $scope.from += $scope.pageSize;
        $vmb.getMails($scope.mailbox.id, $scope.from, $scope.pageSize).then(function(res) {
            $scope.mails = $scope.mails.concat(res.data.mails);
            $scope.totalItems = res.data.paging;
            $scope.$broadcast('scroll.infiniteScrollComplete');
        });
    };

})

.controller('MailCtrl', function($scope, $stateParams, $ionicPopover, $http, $ionicLoading, $ionicModal, $ionicTabsDelegate) {
    $scope.mail = $scope.$parent.selectedMail;
    // .fromTemplate() method


    // .fromTemplateUrl() method
    /* $ionicPopover.fromTemplateUrl('my-popover.html', {
         scope: $scope,
     }).then(function(popover) {
         $scope.popover = popover;
     });*/


    $scope.openScanPopover = function($event) {
        console.log('start popover');
        var template = '<ion-popover-view><ion-header-bar> <h1 class="title">Select a scan method :</h1> </ion-header-bar> <ion-content> <button class="button button-full button-positive" ng-click="scan(\'color\')">Colour</button><button class="button button-full button-dark" ng-click="scan(\'bw\')">Black & White</button></ion-content></ion-popover-view>';
        $scope.popover = $ionicPopover.fromTemplate(template, {
            scope: $scope,
        });
        $scope.popover.show($event);
    };
    $scope.closePopover = function() {
        $scope.popover.hide();
    };
    //Cleanup the popover when we're done with it!
    $scope.$on('$destroy', function() {
        if ($scope.popover) {
            $scope.popover.remove();
        }
    });
    // Execute action on hide popover
    $scope.$on('popover.hidden', function() {
        // Execute action
    });
    // Execute action on remove popover
    $scope.$on('popover.removed', function() {
        // Execute action
    });

    //scan request
    $scope.scan = function(mode) {
        $scope.sendScanRequest(mode);
    };
    $scope.sendScanRequest = function(mode) {
        console.log('start scanning...');
        $http.post('/api/request/create-scan-request', {
            mode: mode,
            mail_id: $scope.mail.id
        }).success(function(res) {
            $ionicLoading.show({
                template: 'Scan request successfully send!',
                noBackdrop: true,
                duration: 2000
            });
            angular.extend($scope.mail, res.mail);
            console.log('Successfully send');
            $scope.popover.hide();
        }).error(function(e) {
            $ionicLoading.show({
                template: 'Something went wrong!',
                noBackdrop: true,
                duration: 2000
            });
            console.log('got some error' + e);
        });
    };

    //forward request
    $scope.openForwardPopover = function($event) {
        console.log('start popover');
        $ionicModal.fromTemplateUrl('templates/ForwardRequest.html', {
            scope: $scope,
            animation: 'slide-in-up'
        }).then(function(modal) {
            $scope.modal = modal;
            $scope.modal.show();
        });
        $scope.selectedAddress = {};
        $http.get('/api/address').success(function(res) {
            $scope.addresses = res;
            $scope.errors = {};
        });
    };
    $scope.forward = function(selectedOption) {
        console.log("start forward request");
        if (selectedOption === 'yes') {
            if ($ionicTabsDelegate.selectedIndex() == 0) {
                $ionicLoading.show({
                    template: 'Please select/enter an address',
                    noBackdrop: true,
                    duration: 2000
                });
                return;
            }

            $ionicTabsDelegate.select(1);
            $http.post('/api/request/create-forward-request', {
                user_id: $scope.user.id,
                mail_id: $scope.mail.id,
                forward_address: $scope.selectedAddress,
            }).success(function(res) {
                // $materialDialog.hide();
                // $scope.showToast('Request successfully send');
                $ionicLoading.show({
                    template: 'Forward request successfully send!',
                    noBackdrop: true,
                    duration: 2000
                });
                angular.extend($scope.mail, res.mail);
                $scope.modal.hide();
            }).error(function(res) {
                if (res.errors.forward_address) {
                    $scope.message = 'All above fields are required';
                    $scope.errors = ' ';
                } else {
                    $scope.errors = res.errors;
                    $scope.message = ' ';
                }
            });
        } else if (selectedOption === 'no') {
            $scope.modal.hide();
        }
    };
    $scope.selectAddress = function(address) {
        angular.extend($scope.selectedAddress, address);
        $ionicTabsDelegate.select(1);
    };
    $scope.hidemodel = function() {
        $scope.modal.hide();
    };

    //Shred request
    $scope.openShredPopover = function($event) {
        console.log('start popover');
        var template = '<ion-popover-view><ion-header-bar> <h1 class="title">Do you really need to Shred ?</h1> </ion-header-bar> <ion-content> <button class="button button-full button-assertive" ng-click="shred(\'Yes\')">Yes</button><button class="button button-full button-dark" ng-click="shred(\'no\')">No</button></ion-content></ion-popover-view>';
        $scope.popover = $ionicPopover.fromTemplate(template, {
            scope: $scope,
        });
        $scope.popover.show($event);
    };
    $scope.shred = function(selectedOption) {
        console.log("start shred request");
        if (selectedOption === 'Yes') {
            $http.post('/api/request/create-shred-request', {
                mail_id: $scope.mail.id
            }).success(function(res) {
                angular.extend($scope.mail, res.mail);
                $scope.popover.hide();
                $ionicLoading.show({
                    template: 'Shred request successfully send!',
                    noBackdrop: true,
                    duration: 2000
                });
            });

        } else if (selectedOption === 'no') {
            $scope.popover.hide();
        }
    };

    //download request
    $scope.download = function(mail) {
        var fileTransfer = new FileTransfer();
        var url = window.__config.API_BASE_URL + '/api/editpdf/generatepdf?mail_id=' + mail.id + '&folder_id=' + mail.folder_id;
        var test_url = "http://unfccc.int/resource/docs/convkp/kpeng.pdf";
        // var ref = window.open(test_url, '_blank', 'location=no');
        // ref.addEventListener('exit', function() {
        //     ref.removeEventListener('exit', function() {});
        //     ref.close();
        // });
        // return;
        var filePath = window.appRootDir.toURL() + "/VMB_Mail.pdf";
        console.log("start download request...");
        fileTransfer.download(url, filePath, function(entry) {
            // alert("download complete: " + entry.toURL());
            window.plugins.socialsharing.share(null, null, entry.toURL());
            // ref = window.open(entry.toURL(), '_blank', 'location=no');
            // ref.addEventListener('exit', function() {
            //     ref.removeEventListener('exit', function() {});
            //     ref.close();
            // });
        }, function(error) {
            console.log(error);
            alert("download error" + error.source);
        }, true);
    };

    var position = 0;
    $scope.move = function(dir) {
        var max = $scope.mails.length;
        if (dir === 'pre') {
            if (position === 0) {
                $scope.currentMail = $scope.mails[0];
            } else {
                $scope.currentMail = $scope.mails[--position];
            }
        } else if (dir === 'next') {
            if (position === max - 1) {
                $scope.currentMail = $scope.mails[max - 1];
            } else {
                $scope.currentMail = $scope.mails[++position];
            }
        }
        $scope.mail = $scope.currentMail;
    };


})

.controller('SearchCtrl', function($scope) {

});
